/*
 * Copyright 2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gradle.testfixtures.internal;

import org.gradle.api.internal.BuildDefinition;
import org.gradle.initialization.DefaultProjectDescriptorRegistry;
import org.gradle.internal.build.BuildState;
import org.gradle.internal.service.Provides;
import org.gradle.internal.service.scopes.BuildScopeServices;

public class TestBuildScopeServices extends BuildScopeServices {

    public TestBuildScopeServices(BuildDefinition buildDefinition, BuildState buildState) {
        super(buildDefinition, buildState);
    }

    @Provides
    protected DefaultProjectDescriptorRegistry createProjectDescriptorRegistry() {
        return new DefaultProjectDescriptorRegistry();
    }
}
