/*
 * Copyright 2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.declarativedsl.dom.mutation

import org.gradle.internal.declarativedsl.dom.DeclarativeDocument


interface DocumentMutationPlan {
    val unsuccessfulDocumentMutations: List<UnsuccessfulDocumentMutation>
}


data class UnsuccessfulDocumentMutation(
    val mutation: DocumentMutation,
    val failureReasons: List<DocumentMutationFailureReason>
)


interface DocumentMutationPlanner<T : DocumentMutationPlan> {
    fun planDocumentMutations(document: DeclarativeDocument, mutations: List<DocumentMutation>): T
}


sealed interface DocumentMutationFailureReason {
    data object TargetNotFoundOrSuperseded : DocumentMutationFailureReason
}
