/*
 * Copyright 2015 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.language.base.internal;

import org.gradle.api.internal.project.ProjectIdentifier;

import java.io.File;

public class ProjectLayout {
    private final ProjectIdentifier projectIdentifier;
    private final File buildDir;

    public ProjectLayout(ProjectIdentifier projectIdentifier, File buildDir) {
        this.projectIdentifier = projectIdentifier;
        this.buildDir = buildDir;
    }

    public ProjectIdentifier getProjectIdentifier() {
        return projectIdentifier;
    }

    public File getBuildDir() {
        return buildDir;
    }
}
