/*
 * Copyright 2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.initialization.buildsrc;

import org.gradle.internal.buildtree.BuildTreeLifecycleController;
import org.gradle.internal.classpath.ClassPath;
import org.jspecify.annotations.NonNull;

public class BuildSrcUpdateFactory {
    private final BuildSrcBuildListenerFactory listenerFactory;

    public BuildSrcUpdateFactory(BuildSrcBuildListenerFactory listenerFactory) {
        this.listenerFactory = listenerFactory;
    }

    @NonNull
    public ClassPath create(BuildTreeLifecycleController buildController) {
        BuildSrcBuildListenerFactory.Listener listener = listenerFactory.create();
        buildController.beforeBuild(gradle -> gradle.addListener(listener));

        buildController.scheduleAndRunTasks(listener);

        return listener.getRuntimeClasspath();
    }
}
