/*
 * Copyright 2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.component.model;

import org.gradle.api.internal.attributes.ImmutableAttributes;

/**
 * The state for a configuration used for graph resolution.
 */
public interface ConfigurationGraphResolveState {
    String getName();

    ImmutableAttributes getAttributes();

    /**
     * Returns a view of this configuration that can be used as a variant.
     */
    VariantGraphResolveState asVariant();

    ConfigurationGraphResolveMetadata getMetadata();
}
